require 'mrlib'

module Redqueen2
   Indent = ' '

   class XMLobject
      attr_reader :name, :attr, :child
      def initialize(name)
         @name = name
         @attr = Hash.new
         @child = []
      end
      def [](key)
         @attr[key]
      end
      def []=(key,val)
         @attr[key] = val
      end
      def each( &block )
         @child.each( &block )
      end
      def <<(obj)
         @child << obj
      end
      def to_s(level = 0)
         tmp =Indent*level + "<#{@name}"
         @attr.each{|k,v|
            tmp += " #{k}=\"#{v}\"" unless v.nil?
         }
         tmp += ">\n"
         @child.each{|x|
            tmp += x.to_s(level + 1)+"\n"
         }
         tmp += ' '*level+"</#{@name}>"
         return tmp
      end
      def fputs(io = STDOUT, level = 0)
         io.print Indent*level + "<#{@name}"
         @attr.each{|k,v|
            io.print " #{k}=\"#{v}\"" unless v.nil?
         }
         io.print ">\n"
         @child.each{|x|
            io.print x.to_s(level + 1)+"\n"
         }
         io.print Indent*level+"</#{@name}>\n"
      end
   end

   class Document < XMLobject
      def initialize
         super('redqueen')
      end
      def to_s
         '<?xml version="1.0" encoding="Shift_JIS" ?>' + "\n" + super(0)
      end
      def fputs(io = STDOUT)
         io.print '<?xml version="1.0" encoding="Shift_JIS" ?>', "\n"
         super(io, 0)
      end
   end

   class Camera < XMLobject
      def initialize(type, width, height, oversampling, fov, blur)
         super('camera')
         @attr['type']		= type
         @attr['width']		= width
         @attr['height']	= height
         @attr['oversampling']	= oversampling
         @attr['fov']		= fov
         @attr['blur']		= blur
      end
   end

   class Renderer < XMLobject
      def initialize(tracedepth, finalgatherray, causticsphotongatherradius, diffusealbedo, diffusedepth)
         super('renderer')
         @attr['tracedepth']			= tracedepth
         @attr['finalgatherray']		= finalgatherray
         @attr['causticsphotongatherradius']	= causticsphotongatherradius
         @attr['diffusealbedo']			= diffusealbedo
         @attr['diffusedepth']			= diffusedepth
      end
   end

   class Sky < XMLobject
      def initialize(type)
         super('sky')
         @attr['type'] = type
      end
   end

   class Material < XMLobject
      def initialize(name)
         super('material')
         @attr['name'] = name
      end
   end

   class Object < XMLobject
      def initialize(name, smoothing)
         super('object')
         @attr['name'] = name
         @attr['smoothing'] = smoothing
      end
   end

   class Part < XMLobject
      def initialize(name, material)
         super('part')
         @attr['name'] = name
         @attr['material'] = material
      end
   end

   class Vertices < XMLobject
      def initialize
         super('vertices')
      end
   end

   class Triangles < XMLobject
      def initialize
         super('triangles')
      end
   end

   class Vector < MetaRed::Vector
      attr_accessor :name
      def initialize(x, y, z, name = 'vector')
         @name = name
         @x, @y, @z = x, y, z
      end
      def to_s(level = 0)
         Indent*level+"<#{@name} x=\"#{@x}\" y=\"#{@y}\" z=\"#{@z}\" />"
      end
      def fputs(io = STDOUT, level = 0)
         io.puts self.to_s(level)
      end
   end

   class Vertex < Vector
      def initialize(x, y, z, u = nil, v = nil)
         @x, @y, @z = x, y, z
         @u, @v = u, v
      end
      def to_s(level = 0)
         tmp = Indent*level+"<vertex x=\"#{@x}\" y=\"#{@y}\" z=\"#{@z}\" "
         tmp += " u=\"#{@u}\" v=\"#{@v}\" " if !@u.nil? && !@v.nil?
         tmp += "/>"
         return tmp
      end
      def fputs(io = STDOUT, level = 0)
         io.puts self.to_s(level)
      end
   end

   class Color < Vector
      def initialize(x, y, z, name = 'color')
         @x, @y, @z, @name = x, y, z, name
      end
      def to_s(level = 0)
         Indent*level+"<#{@name} r=\"#{@x}\" g=\"#{@y}\" b=\"#{@z}\" />"
      end
      alias r x
      alias g y
      alias b z
      alias r= x=
      alias g= y=
      alias b= z=
      def fputs(io = STDOUT, level = 0)
         io.puts self.to_s(level)
      end
   end

   class Triangle
      attr_accessor :id0, :id1, :id2
      def initialize(id0, id1, id2)
         @id0, @id1, @id2 = id0, id1, id2
      end
      def to_s(level = 0)
         Indent*level+"<triangle id0=\"#{@id0}\" id1=\"#{@id1}\" id2=\"#{@id2}\" />"
      end
      def fputs(io = STDOUT, level = 0)
         io.puts self.to_s(level)
      end
   end

   class Image
      attr_accessor :file
      def initialize(file)
         @file = file
      end
      def to_s(level = 0)
         Indent*level+"<image name=\"#{@file}\" />"
      end
      def fputs(io = STDOUT, level = 0)
         io.puts self.to_s(level)
      end
   end

   class ParameterError < StandardError
   end

   class Light < XMLobject
   end

   class ParallelLight < Light
      def initialize(color, direction, photon = 50000, blur = 1)
         raise ParameterError unless color.kind_of? Color
         raise ParameterError unless direction.kind_of? Vector
         super('parallellight')
         @attr['photon'] = photon
         @attr['blur'] = blur
         @child << Color.new(color.r, color.g, color.b, 'color')
         @child << Vector.new(direction.x, direction.y, direction.z, 'direction')
      end
   end
   class PointLight < Light
      def initialize(color, location, photon = 50000)
         raise ParameterError unless color.kind_of? Color
         raise ParameterError unless location.kind_of? Vector
         super('pointlight')
         @attr['photon'] = photon
         @child << Color.new(color.r, color.g, color.b, 'color')
         @child << Vector.new(location.x, location.y, location.z, 'location')
      end
   end
   class SphereLight < Light
      def initialize(color, location, photon = 50000, radius = 1, shadowray = 16)
         raise ParameterError unless color.kind_of? Color
         raise ParameterError unless location.kind_of? Vector
         super('spherelight')
         @attr['photon'] = photon
         @attr['radius'] = radius
         @attr['shadowray'] = shadowray
         @child << Color.new(color.r, color.g, color.b, 'color')
         @child << Vector.new(location.x, location.y, location.z, 'location')
      end
   end
   class AreaLight < Light
      def initialize(color, location, vector_u, vector_v, photon = 50000, shadowray = 100)
         raise ParameterError unless color.kind_of? Color
         raise ParameterError unless location.kind_of? Vector
         raise ParameterError unless vector_u.kind_of? Vector
         raise ParameterError unless vector_v.kind_of? Vector
         super('arealight')
         @attr['photon'] = photon
         @attr['shadowray'] = shadowray
         @child << Color.new(color.r, color.g, color.b, 'color')
         @child << Vector.new(location.x, location.y, location.z, 'location')
         @child << Vector.new(vector_u.x, vector_u.y, vector_u.z, 'vectoru')
         @child << Vector.new(vector_v.x, vector_v.y, vector_v.z, 'vectorv')
      end
   end
end

#= PƂŌĂяoꂽꍇ̂ݗLȏB
# CuƂČĂяoꂽꍇɂ͖B
if $0 == __FILE__
   rq2 = Redqueen2::Document.new
   c = Redqueen2::Camera.new('pinhole', 640, 480, 12, 21, 1)
   rq2.child << c
   c.child << Redqueen2::Vector.new(1,1,1,'location')
   c.child << Redqueen2::Vector.new(1,1,1,'upvector')
   rq2.fputs
end
